@@ -53,11 +53,11 @@ App::Dochazka::CLI - Dochazka command line client
 
 =head1 VERSION
 
-Version 0.154
+Version 0.184
 
 =cut
 
-our $VERSION = '0.154';
+our $VERSION = '0.184';
 
 
 
@@ -82,9 +82,9 @@ It implements a number of commands, which are documented in L<App::Dochazka::CLI
 
 =over
 
-=item * C<< $current_date >>
+=item * C<< $prompt_date >>
 
-The current date, as determined by C<bin/dochazka-cli>
+The date displayed in the prompt -- see C<bin/dochazka-cli>
 
 =item * C<< $current_emp >>
 
@@ -102,9 +102,9 @@ Tells parser whether to display debug messages
 
 =cut
 
-our ( $current_date, $current_emp, $current_priv, $debug_mode, $message );
+our ( $prompt_date, $current_emp, $current_priv, $debug_mode, $message );
 our @EXPORT_OK = qw( 
-    $current_date 
+    $prompt_date 
     $current_emp 
     $current_priv 
     $debug_mode
