@@ -56,6 +56,19 @@ use App::Dochazka::CLI::Commands::History qw(
     dump_schedule_history
     set_history_remark
 );
+use App::Dochazka::CLI::Commands::Interval qw(
+    interval_fetch_date
+    interval_fetch_date_date1
+    interval_fetch_month
+    interval_fetch_num_num1
+    interval_fetch_promptdate
+    interval_new_date_time_date1_time1
+    interval_new_time_time1
+    interval_new_timerange
+);
+use App::Dochazka::CLI::Commands::Misc qw( 
+    change_prompt_date 
+);
 use App::Dochazka::CLI::Commands::Priv qw(
     show_current_priv
 );
@@ -85,11 +98,11 @@ App::Dochazka::CLI::CommandMap - Command map
 
 =head1 VERSION
 
-Version 0.154
+Version 0.184
 
 =cut
 
-our $VERSION = '0.154';
+our $VERSION = '0.184';
 
 
 
@@ -323,6 +336,18 @@ my $dispatch_map = {
     "PUT INTERVAL EID _NUM _TSRANGE" => \&_method_interval_eid_tsrange,
     "POST INTERVAL EID _NUM _TSRANGE" => \&_method_interval_eid_tsrange,
     "DELETE INTERVAL EID _NUM _TSRANGE" => \&_method_interval_eid_tsrange,
+    "GET INTERVAL FILLUP EID _NUM _DATE _DATE1" => \&_method_interval_fillup_eid,
+    "PUT INTERVAL FILLUP EID _NUM _DATE _DATE1" => \&_method_interval_fillup_eid,
+    "POST INTERVAL FILLUP EID _NUM _DATE _DATE1" => \&_method_interval_fillup_eid,
+    "DELETE INTERVAL FILLUP EID _NUM _DATE _DATE1" => \&_method_interval_fillup_eid,
+    "GET INTERVAL FILLUP NICK _TERM _DATE _DATE1" => \&_method_interval_fillup_nick,
+    "PUT INTERVAL FILLUP NICK _TERM _DATE _DATE1" => \&_method_interval_fillup_nick,
+    "POST INTERVAL FILLUP NICK _TERM _DATE _DATE1" => \&_method_interval_fillup_nick,
+    "DELETE INTERVAL FILLUP NICK _TERM _DATE _DATE1" => \&_method_interval_fillup_nick,
+    "GET INTERVAL FILLUP SELF _DATE _DATE1" => \&_method_interval_fillup_self,
+    "PUT INTERVAL FILLUP SELF _DATE _DATE1" => \&_method_interval_fillup_self,
+    "POST INTERVAL FILLUP SELF _DATE _DATE1" => \&_method_interval_fillup_self,
+    "DELETE INTERVAL FILLUP SELF _DATE _DATE1" => \&_method_interval_fillup_self,
     "GET INTERVAL IID _NUM" => \&_method_interval_iid,
     "PUT INTERVAL IID _NUM" => \&_method_interval_iid,
     "POST INTERVAL IID _NUM" => \&_method_interval_iid,
@@ -396,8 +421,8 @@ my $dispatch_map = {
     "POST PRIV EID _NUM" => \&_method_priv_eid_num,
     "DELETE PRIV EID _NUM" => \&_method_priv_eid_num,
     "GET PRIV EID _NUM _TIMESTAMP" => \&_method_priv_eid_num_timestamp,
-    "PUT PRIV EID _NUM _TIMESTAMP" => \&_method_priv_eid_num_timestamp,
-    "POST PRIV EID _NUM _TIMESTAMP" => \&_method_priv_eid_num_timestamp,
+    "PUT PRIV EID _NUM _TIMESTAMP" => \&_method_priv_eid_num_timestamp, 
+    "POST PRIV EID _NUM _TIMESTAMP" => \&_method_priv_eid_num_timestamp, 
     "DELETE PRIV EID _NUM _TIMESTAMP" => \&_method_priv_eid_num_timestamp,
     "GET PRIV HISTORY EID _NUM" => \&_method_priv_history_eid_num,
     "PUT PRIV HISTORY EID _NUM" => \&_method_priv_history_eid_num,
@@ -543,10 +568,101 @@ my $dispatch_map = {
     "EMPLOYEE_SPEC PASSWORD" => \&set_employee_other_password,
     "EMPLOYEE_SPEC SET PASSWORD" => \&set_employee_other_password,
 
+    # History commands
+    "PRIV HISTORY" => \&dump_priv_history,
+    "EMPLOYEE_SPEC PRIV HISTORY" => \&dump_priv_history,
+    "SCHEDULE HISTORY" => \&dump_schedule_history,
+    "EMPLOYEE_SPEC SCHEDULE HISTORY" => \&dump_schedule_history,
+    "EMPLOYEE_SPEC PRIV_SPEC _DATE" => \&add_priv_history,
+    "EMPLOYEE_SPEC PRIV_SPEC EFFECTIVE _DATE" => \&add_priv_history,
+    "EMPLOYEE_SPEC SCHEDULE_SPEC _DATE" => \&add_schedule_history,
+    "EMPLOYEE_SPEC SCHEDULE_SPEC EFFECTIVE _DATE" => \&add_schedule_history,
+    "EMPLOYEE_SPEC SET PRIV_SPEC _DATE" => \&add_priv_history,
+    "EMPLOYEE_SPEC SET PRIV_SPEC EFFECTIVE _DATE" => \&add_priv_history,
+    "EMPLOYEE_SPEC SET SCHEDULE_SPEC _DATE" => \&add_schedule_history,
+    "EMPLOYEE_SPEC SET SCHEDULE_SPEC EFFECTIVE _DATE" => \&add_schedule_history,
+    "PHISTORY_SPEC REMARK" => \&set_history_remark,
+    "PHISTORY_SPEC SET REMARK" => \&set_history_remark,
+    "SHISTORY_SPEC REMARK" => \&set_history_remark,
+    "SHISTORY_SPEC SET REMARK" => \&set_history_remark,
+
+    # Interval commands
+
+    # fetch/fillup intervals
+    "INTERVAL" => \&interval_fetch_promptdate,
+    "EMPLOYEE_SPEC INTERVAL" => \&interval_fetch_promptdate,
+    "INTERVAL FETCH" => \&interval_fetch_promptdate,
+    "EMPLOYEE_SPEC INTERVAL FETCH" => \&interval_fetch_promptdate,
+    "INTERVAL FILLUP" => \&interval_fetch_promptdate,
+    "EMPLOYEE_SPEC INTERVAL FILLUP" => \&interval_fetch_promptdate,
+
+    "INTERVAL _DATE" => \&interval_fetch_date,
+    "EMPLOYEE_SPEC INTERVAL _DATE" => \&interval_fetch_date,
+    "INTERVAL FETCH _DATE" => \&interval_fetch_date,
+    "EMPLOYEE_SPEC INTERVAL FETCH _DATE" => \&interval_fetch_date,
+    "INTERVAL FILLUP _DATE" => \&interval_fetch_date,
+    "EMPLOYEE_SPEC INTERVAL FILLUP _DATE" => \&interval_fetch_date,
+
+    "INTERVAL _DATE _DATE1" => \&interval_fetch_date_date1,
+    "EMPLOYEE_SPEC INTERVAL _DATE _DATE1" => \&interval_fetch_date_date1,
+    "INTERVAL FETCH _DATE _DATE1" => \&interval_fetch_date_date1,
+    "EMPLOYEE_SPEC INTERVAL FETCH _DATE _DATE1" => \&interval_fetch_date_date1,
+    "INTERVAL FILLUP _DATE _DATE1" => \&interval_fetch_date_date1,
+    "EMPLOYEE_SPEC INTERVAL FILLUP _DATE _DATE1" => \&interval_fetch_date_date1,
+    "INTERVAL _DATE _HYPHEN _DATE1" => \&interval_fetch_date_date1,
+    "EMPLOYEE_SPEC INTERVAL _DATE _HYPHEN _DATE1" => \&interval_fetch_date_date1,
+    "INTERVAL FETCH _DATE _HYPHEN _DATE1" => \&interval_fetch_date_date1,
+    "EMPLOYEE_SPEC INTERVAL FETCH _DATE _HYPHEN _DATE1" => \&interval_fetch_date_date1,
+    "INTERVAL FILLUP _DATE _HYPHEN _DATE1" => \&interval_fetch_date_date1,
+    "EMPLOYEE_SPEC INTERVAL FILLUP _DATE _HYPHEN _DATE1" => \&interval_fetch_date_date1,
+
+    "INTERVAL _MONTH" => \&interval_fetch_month,
+    "EMPLOYEE_SPEC INTERVAL _MONTH" => \&interval_fetch_month,
+    "INTERVAL FETCH _MONTH" => \&interval_fetch_month,
+    "EMPLOYEE_SPEC INTERVAL FETCH _MONTH" => \&interval_fetch_month,
+    "INTERVAL FILLUP _MONTH" => \&interval_fetch_month,
+    "EMPLOYEE_SPEC INTERVAL FILLUP _MONTH" => \&interval_fetch_month,
+    "INTERVAL _MONTH _NUM" => \&interval_fetch_month,
+    "EMPLOYEE_SPEC INTERVAL _MONTH _NUM" => \&interval_fetch_month,
+    "INTERVAL FETCH _MONTH _NUM" => \&interval_fetch_month,
+    "EMPLOYEE_SPEC INTERVAL FETCH _MONTH _NUM" => \&interval_fetch_month,
+    "INTERVAL FILLUP _MONTH _NUM" => \&interval_fetch_month,
+    "EMPLOYEE_SPEC INTERVAL FILLUP _MONTH _NUM" => \&interval_fetch_month,
+
+    "INTERVAL _NUM" => \&interval_fetch_num_num1,
+    "EMPLOYEE_SPEC INTERVAL _NUM" => \&interval_fetch_num_num1,
+    "INTERVAL FETCH _NUM" => \&interval_fetch_num_num1,
+    "EMPLOYEE_SPEC INTERVAL FETCH _NUM" => \&interval_fetch_num_num1,
+    "INTERVAL FILLUP _NUM" => \&interval_fetch_num_num1,
+    "EMPLOYEE_SPEC INTERVAL FILLUP _NUM" => \&interval_fetch_num_num1,
+    "INTERVAL _NUM _NUM1" => \&interval_fetch_num_num1,
+    "EMPLOYEE_SPEC INTERVAL _NUM _NUM1" => \&interval_fetch_num_num1,
+    "INTERVAL FETCH _NUM _NUM1" => \&interval_fetch_num_num1,
+    "EMPLOYEE_SPEC INTERVAL FETCH _NUM _NUM1" => \&interval_fetch_num_num1,
+    "INTERVAL FILLUP _NUM _NUM1" => \&interval_fetch_num_num1,
+    "EMPLOYEE_SPEC INTERVAL FILLUP _NUM _NUM1" => \&interval_fetch_num_num1,
+
+    # add/insert new intervals
+    "INTERVAL _TIME _TIME1 _TERM" => \&interval_new_time_time1,
+    "INTERVAL _TIME _HYPHEN _TIME1 _TERM" => \&interval_new_time_time1,
+    "INTERVAL _TIMERANGE _TERM" => \&interval_new_timerange,
+    "INTERVAL _DATE _TIME _TIME1 _TERM" => \&interval_new_time_time1,
+    "INTERVAL _DATE _TIME _HYPHEN _TIME1 _TERM" => \&interval_new_time_time1,
+    "INTERVAL _DATE _TIMERANGE _TERM" => \&interval_new_timerange,
+    "INTERVAL _DATE _TIME _DATE1 _TIME1 _TERM" => \&interval_new_date_time_date1_time1,
+    "INTERVAL _DATE _TIME _HYPHEN _DATE _TIME1 _TERM" => \&interval_new_date_time_date1_time1,
+
+
+    # Lock commands
+
     # Priv commands
     "PRIV" => \&show_current_priv,
     "EMPLOYEE_SPEC PRIV" => \&show_current_priv,
 
+    # Prompt date commands
+    "PROMPT _DATE" => \&change_prompt_date,
+    "PROMPT DATE _DATE" => \&change_prompt_date,
+
     # Schedule commands
     "SCHEDULE" => \&show_current_schedule,
     "EMPLOYEE_SPEC SCHEDULE" => \&show_current_schedule,
@@ -568,24 +684,6 @@ my $dispatch_map = {
     "SCHEDULE_SPEC SET REMARK" => \&schedulespec_remark,
     "SCHEDULE_SPEC SET SCODE _TERM" => \&schedulespec_scode,
 
-    # History commands
-    "PRIV HISTORY" => \&dump_priv_history,
-    "EMPLOYEE_SPEC PRIV HISTORY" => \&dump_priv_history,
-    "SCHEDULE HISTORY" => \&dump_schedule_history,
-    "EMPLOYEE_SPEC SCHEDULE HISTORY" => \&dump_schedule_history,
-    "EMPLOYEE_SPEC PRIV_SPEC _TIMESTAMP" => \&add_priv_history,
-    "EMPLOYEE_SPEC PRIV_SPEC EFFECTIVE _TIMESTAMP" => \&add_priv_history,
-    "EMPLOYEE_SPEC SCHEDULE_SPEC _TIMESTAMP" => \&add_schedule_history,
-    "EMPLOYEE_SPEC SCHEDULE_SPEC EFFECTIVE _TIMESTAMP" => \&add_schedule_history,
-    "EMPLOYEE_SPEC SET PRIV_SPEC _TIMESTAMP" => \&add_priv_history,
-    "EMPLOYEE_SPEC SET PRIV_SPEC EFFECTIVE _TIMESTAMP" => \&add_priv_history,
-    "EMPLOYEE_SPEC SET SCHEDULE_SPEC _TIMESTAMP" => \&add_schedule_history,
-    "EMPLOYEE_SPEC SET SCHEDULE_SPEC EFFECTIVE _TIMESTAMP" => \&add_schedule_history,
-    "PHISTORY_SPEC REMARK" => \&set_history_remark,
-    "PHISTORY_SPEC SET REMARK" => \&set_history_remark,
-    "SHISTORY_SPEC REMARK" => \&set_history_remark,
-    "SHISTORY_SPEC SET REMARK" => \&set_history_remark,
-
 };
 
 our $semantic_tree = {};
