@@ -39,8 +39,13 @@ use warnings;
 
 use App::CELL qw( $CELL );
 use App::Dochazka::CLI qw( $current_emp $debug_mode );
-use App::Dochazka::CLI::Util qw( lookup_employee refresh_current_emp rest_error );
-use App::Dochazka::REST::Model::Employee;
+use App::Dochazka::CLI::Util qw( 
+    determine_employee
+    parse_test 
+    refresh_current_emp 
+    rest_error 
+);
+use App::Dochazka::Model::Employee;
 use Data::Dumper;
 use Exporter 'import';
 use Web::MREST::CLI::UserAgent qw( send_req );
@@ -57,11 +62,11 @@ App::Dochazka::CLI::Commands::Employee - Employee commands
 
 =head1 VERSION
 
-Version 0.154
+Version 0.184
 
 =cut
 
-our $VERSION = '0.154';
+our $VERSION = '0.184';
 
 
 
@@ -82,7 +87,9 @@ our @EXPORT_OK = qw(
 
 =head1 FUNCTIONS
 
-=head2 employee_profile
+=head2 Command handlers
+
+=head3 employee_profile
 
     EMPLOYEE
     EMPLOYEE_SPEC
@@ -96,20 +103,14 @@ our @EXPORT_OK = qw(
 sub employee_profile {
     print "Entering " . __PACKAGE__ . "::employee_profile\n" if $debug_mode;
     my ( $ts, $th ) = @_;
-    print "ts " . Dumper( $ts ) if $debug_mode;
-    print "th " . Dumper( $th ) if $debug_mode;
-    my $s_key = $th->{'EMPLOYEE_SPEC'};
 
-    print "Search key: " . Dumper( $s_key ) . "\n" if $debug_mode;
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
 
-    die "AAAAGAGAAGGGGHHGHHHH!" if defined( $s_key ) and ref( $s_key ) ne '';
-
-    # GET employee/eid/:eid
-    my $status = ( $s_key )
-        ? lookup_employee( $s_key )
-        : refresh_current_emp();
-    return rest_error( $status, "Employee lookup" ) unless $status->ok;
-    my $emp = App::Dochazka::REST::Model::Employee->spawn( %{ $status->payload } );
+    # determine employee
+    my $status = determine_employee( $th->{'EMPLOYEE_SPEC'} );
+    return $status unless $status->ok;
+    my $emp = $status->payload;
 
     # GET priv/eid/:eid
     $status = send_req( 'GET', 'priv/eid/' . $emp->eid );
@@ -131,7 +132,7 @@ sub employee_profile {
     return $CELL->status_ok( 'DOCHAZKA_CLI_NORMAL_COMPLETION', payload => $message );
 }
 
-=head2 set_employee_self_sec_id
+=head3 set_employee_self_sec_id
 
 SET EMPLOYEE SEC_ID _TERM
 
@@ -139,7 +140,11 @@ SET EMPLOYEE SEC_ID _TERM
 
 sub set_employee_self_sec_id {
     my ( $ts, $th ) = @_;
-    return set_employee( 
+
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
+
+    return _set_employee( 
         emp_obj => $current_emp,
         prop => 'sec_id', 
         val => $th->{'_TERM'},
@@ -147,7 +152,7 @@ sub set_employee_self_sec_id {
 }
 
 
-=head2 set_employee_self_fullname
+=head3 set_employee_self_fullname
 
 SET EMPLOYEE FULLNAME
 
@@ -155,7 +160,11 @@ SET EMPLOYEE FULLNAME
 
 sub set_employee_self_fullname {
     my ( $ts, $th ) = @_;
-    return set_employee( 
+
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
+
+    return _set_employee( 
         emp_obj => $current_emp,
         prop => 'fullname', 
         val => $th->{'_REST'},
@@ -163,7 +172,7 @@ sub set_employee_self_fullname {
 }
 
 
-=head2 set_employee_other_sec_id
+=head3 set_employee_other_sec_id
 
 EMPLOYEE_SPEC SET SEC_ID _TERM
 
@@ -171,7 +180,11 @@ EMPLOYEE_SPEC SET SEC_ID _TERM
 
 sub set_employee_other_sec_id {
     my ( $ts, $th ) = @_;
-    return set_employee( 
+
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
+
+    return _set_employee( 
         emp_spec => $th->{'EMPLOYEE_SPEC'},
         prop => 'sec_id', 
         val => $th->{'_TERM'},
@@ -179,7 +192,7 @@ sub set_employee_other_sec_id {
 }
 
 
-=head2 set_employee_other_fullname
+=head3 set_employee_other_fullname
 
 EMPLOYEE_SPEC SET FULLNAME
 
@@ -187,7 +200,11 @@ EMPLOYEE_SPEC SET FULLNAME
 
 sub set_employee_other_fullname {
     my ( $ts, $th ) = @_;
-    return set_employee( 
+
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
+
+    return _set_employee( 
         emp_spec => $th->{'EMPLOYEE_SPEC'},
         prop => 'fullname', 
         val => $th->{'_REST'},
@@ -195,18 +212,75 @@ sub set_employee_other_fullname {
 }
 
 
-=head2 set_employee
+=head3 set_employee_self_password
+
+Reset one's own password
+
+    EMPLOYEE PASSWORD
+    EMPLOYEE SET PASSWORD
 
 =cut
 
-sub set_employee {
+sub set_employee_self_password {
+    my ( $ts, $th ) = @_;
+
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
+
+    return _set_password( 
+        eid => $current_emp->eid,
+        password => $th->{'_REST'},
+    ); 
+}
+
+
+=head3 set_employee_other_password
+
+Reset password of an arbitrary employee
+
+    EMPLOYEE_SPEC PASSWORD
+    EMPLOYEE_SPEC SET PASSWORD
+
+=cut
+
+sub set_employee_other_password {
+    print "Entering " . __PACKAGE__ . "::set_employee_other_password\n" if $debug_mode;
+    my ( $ts, $th ) = @_;
+
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
+
+    my $status = determine_employee( $th->{EMPLOYEE_SPEC} );
+    return $status unless $status->ok;
+    my $emp = $status->payload;
+
+    return _set_password( 
+        eid => $emp->eid,
+        password => $th->{'_REST'},
+    ); 
+}
+
+
+
+=head2 Helper functions
+
+Functions used by multiple handlers
+
+
+=head3 _set_employee
+
+Function that the handlers are wrappers of
+
+=cut
+
+sub _set_employee {
     my %PROPLIST = @_;
     my $status;
     my $emp_obj;
     if ( my $e_spec = $PROPLIST{'emp_spec'} ) {
-        $status = lookup_employee( $e_spec );
-        return rest_error( $status, "Employee lookup" ) unless $status->ok;
-        $emp_obj = App::Dochazka::REST::Model::Employee->spawn( %{ $status->payload } );
+        $status = determine_employee( $e_spec );
+        return $status unless $status->ok;
+        $emp_obj = $status->payload;
     } elsif ( $emp_obj = $PROPLIST{'emp_obj'} ) {
     } else {
         die "AAAAAAAAAAAAAHHHHH!";
@@ -227,44 +301,35 @@ EOS
 }
 
 
-=head2 set_employee_self_password
+=head3 _set_password
 
-Reset one's own password
+Takes PARAMHASH with following properties:
+
+     eid => EID of employee
+     password => the new password (*optional*)
 
 =cut
 
-sub set_employee_self_password {
-    return _set_password( $current_emp->eid ); 
-}
-
-
-=head2 set_employee_other_password
-
-Reset password of an arbitrary employee
-
-=cut
-
-sub set_employee_other_password {
-    print "Entering " . __PACKAGE__ . "::set_employee_other_password\n" if $debug_mode;
-    my ( $ts, $th ) = @_;
-
-    my $status = lookup_employee( $th->{EMPLOYEE_SPEC} );
-    return rest_error( $status, "Employee lookup" ) unless $status->ok;
-    my $emp_obj = App::Dochazka::REST::Model::Employee->spawn( %{ $status->payload } );
-
-    return _set_password( $emp_obj->eid );
-}
-
-
-# take EID parameter
 sub _set_password {
-    my $eid = shift;
+    my %PH = @_;
+    my $eid = $PH{'eid'};
+    my $newpass = $PH{'password'};
 
-    # prompt for new password and take basic precautions
-    print "New password: ";
-    my $newpass = <>;
-    chomp( $newpass );
-    print "Again: ";
+    print "It is important that the new password really be what you intended.\n";
+    print "Therefore, we are going to ask you to enter the desired password\n";
+    print "twice, so you have a chance to double-check. ";
+    print "\n\n";
+
+    # prompt for new password and ask nicely for confirmation
+    if ( length( $newpass ) > 0 ) {
+        print "New password      : $newpass\n";
+    } else {
+        print "New password      : ";
+        $newpass = <>;
+        chomp( $newpass );
+    }
+    
+    print "New password again: ";
     my $confirm = <>;
     chomp( $confirm );
     if ( $newpass ne $confirm ) {
@@ -276,7 +341,9 @@ sub _set_password {
 { "password" : "$newpass" }
 EOS
 
-    return $status;
+    return $status unless $status->ok;
+    return $CELL->status_ok( 'DOCHAZKA_CLI_NORMAL_COMPLETION', 
+        payload => "Password changed" );
 }
 
 
