@@ -39,7 +39,7 @@ use warnings;
 
 use App::CELL qw( $CELL );
 use App::Dochazka::CLI qw( $current_emp $debug_mode );
-use App::Dochazka::CLI::Util qw( lookup_employee rest_error truncate_to );
+use App::Dochazka::CLI::Util qw( lookup_employee parse_test rest_error truncate_to );
 use Data::Dumper;
 use Exporter 'import';
 use Text::Table;
@@ -57,11 +57,11 @@ App::Dochazka::CLI::Commands::Activity - Activity commands
 
 =head1 VERSION
 
-Version 0.154
+Version 0.184
 
 =cut
 
-our $VERSION = '0.154';
+our $VERSION = '0.184';
 
 
 
@@ -85,19 +85,80 @@ our @EXPORT_OK = qw(
 
 The functions in this module are called from the parser when it recognizes a command.
 
+
+=head2 Command handlers
+
+Command handler functions are called from the parser.
+
+
+=head3 dump_priv_history
+
+    PRIV HISTORY
+    EMPLOYEE_SPEC PRIV HISTORY
+
 =cut
 
+sub dump_priv_history {
+    print "Entering " . __PACKAGE__ . "::dump_priv_history\n" if $debug_mode;
+    my ( $ts, $th ) = @_;
 
-=head2 add_priv_history
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
+
+    my $emp_spec = ( $th->{'EMPLOYEE_SPEC'} )
+        ? $th->{'EMPLOYEE_SPEC'}
+        : $current_emp;
+
+    return _dump_history(
+        emp_spec => $emp_spec,
+        type => 'priv',
+    );
+}
+
+
+=head3 dump_schedule_history
+
+    SCHEDULE HISTORY
+    EMPLOYEE_SPEC SCHEDULE HISTORY
+
+=cut
+
+sub dump_schedule_history {
+    print "Entering " . __PACKAGE__ . "::dump_schedule_history\n" if $debug_mode;
+    my ( $ts, $th ) = @_;
+
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
+
+    my $emp_spec = ( $th->{'EMPLOYEE_SPEC'} )
+        ? $th->{'EMPLOYEE_SPEC'}
+        : $current_emp;
+
+    return _dump_history(
+        emp_spec => $emp_spec,
+        type => 'schedule',
+    );
+}
+
+
+=head3 add_priv_history
 
 Add privilege history record.
 
+    EMPLOYEE_SPEC PRIV_SPEC _DATE
+    EMPLOYEE_SPEC PRIV_SPEC EFFECTIVE _DATE
+    EMPLOYEE_SPEC SET PRIV_SPEC _DATE
+    EMPLOYEE_SPEC SET PRIV_SPEC EFFECTIVE _DATE
+
 =cut
 
 sub add_priv_history {
     print "Entering " . __PACKAGE__ . "::add_priv_history\n" if $debug_mode;
     my ( $ts, $th ) = @_;
 
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
+
     my $emp_spec = ( $th->{'EMPLOYEE_SPEC'} )
         ? $th->{'EMPLOYEE_SPEC'}
         : $current_emp;
@@ -105,36 +166,97 @@ sub add_priv_history {
     return _add_history(
         emp_spec => $emp_spec,
         type => 'priv',
-        effective => $th->{'_TIMESTAMP'},
+        effective => $th->{'_DATE'},
         priv => $th->{'PRIV_SPEC'},
     );
 }
 
-=head2 add_schedule_history
+
+=head3 add_schedule_history
 
 Add schedule history record.
 
-=cut
+    EMPLOYEE_SPEC SCHEDULE_SPEC _DATE
+    EMPLOYEE_SPEC SCHEDULE_SPEC EFFECTIVE _DATE
+    EMPLOYEE_SPEC SET SCHEDULE_SPEC _DATE
+    EMPLOYEE_SPEC SET SCHEDULE_SPEC EFFECTIVE _DATE
+
+=cut 
 
 sub add_schedule_history {
     print "Entering " . __PACKAGE__ . "::add_schedule_history\n" if $debug_mode;
     my ( $ts, $th ) = @_;
 
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
+
     my $emp_spec = ( $th->{'EMPLOYEE_SPEC'} )
         ? $th->{'EMPLOYEE_SPEC'}
         : $current_emp;
 
     my ( $key_spec, $key ) = $th->{'SCHEDULE_SPEC'} =~ m/^(.*)\=(.*)$/;
+    $key_spec = lc $key_spec;
 
     return _add_history(
         emp_spec => $emp_spec,
         type => 'schedule',
-        effective => $th->{'_TIMESTAMP'},
+        effective => $th->{'_DATE'},
         $key_spec => $key,
     );
 }
 
-# add a history record
+
+=head3 set_history_remark
+
+    PHISTORY_SPEC REMARK
+    PHISTORY_SPEC SET REMARK
+    SHISTORY_SPEC REMARK
+    SHISTORY_SPEC SET REMARK
+
+=cut
+
+sub set_history_remark {
+    print "Entering " . __PACKAGE__ . "::dump_schedule_history\n" if $debug_mode;
+    my ( $ts, $th ) = @_;
+
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
+
+    my ( $type, $id_spec, $id );
+    if ( $th->{'PHISTORY_SPEC'} ) {
+        $type = 'priv';
+        $id_spec = 'phid';
+        $id = $th->{'PHISTORY_SPEC'};
+    } elsif ( $th->{'SHISTORY_SPEC'} ) {
+        $type = 'schedule';
+        $id_spec = 'shid';
+        $id = $th->{'SHISTORY_SPEC'};
+    } else {
+        die "Agh~! neither PHISTORY_SPEC nor SHISTORY_SPEC given";
+    }
+
+    my $remark = $th->{'_REST'} || '';
+    $remark =~ s/[\"\']//g;
+
+    my $status = send_req( 'POST', "$type/history/$id_spec/$id", <<"EOS" );
+{ "remark" : "$remark" }
+EOS
+
+    return $status;
+}
+
+
+=head2 Helper functions
+
+Functions called from multiple command handlers.
+
+
+=head3 _add_history
+
+Add a history record
+
+=cut
+
 sub _add_history {
     my %ARGS = @_;
 
@@ -143,7 +265,7 @@ sub _add_history {
 
     # get EID
     my ( $emp_obj, $status ); 
-    $status = process_employee_spec( $ARGS{'emp_spec'} );
+    $status = _process_employee_spec( $ARGS{'emp_spec'} );
     return $status unless $status->ok;
     my $eid = $status->payload->{'eid'};
 
@@ -172,7 +294,7 @@ sub _add_history {
     }
 
     # get effective
-    my $effective = $ARGS{'effective'};
+    my $effective = $ARGS{'effective'} || "undefined";
 
     # send REST request
     $status = send_req( 'POST', "$type/history/eid/$eid", <<"EOS" );
@@ -195,44 +317,10 @@ EOS
 }
 
 
-=head2 dump_priv_history
+=head3 _dump_history
 
 =cut
 
-sub dump_priv_history {
-    print "Entering " . __PACKAGE__ . "::dump_priv_history\n" if $debug_mode;
-    my ( $ts, $th ) = @_;
-
-    my $emp_spec = ( $th->{'EMPLOYEE_SPEC'} )
-        ? $th->{'EMPLOYEE_SPEC'}
-        : $current_emp;
-
-    return _dump_history(
-        emp_spec => $emp_spec,
-        type => 'priv',
-    );
-}
-
-
-=head2 dump_schedule_history
-
-=cut
-
-sub dump_schedule_history {
-    print "Entering " . __PACKAGE__ . "::dump_schedule_history\n" if $debug_mode;
-    my ( $ts, $th ) = @_;
-
-    my $emp_spec = ( $th->{'EMPLOYEE_SPEC'} )
-        ? $th->{'EMPLOYEE_SPEC'}
-        : $current_emp;
-
-    return _dump_history(
-        emp_spec => $emp_spec,
-        type => 'schedule',
-    );
-}
-
-
 sub _dump_history {
     my %ARGS = @_;
 
@@ -241,7 +329,7 @@ sub _dump_history {
 
     # get EID
     my ( $emp_obj, $status ); 
-    $status = process_employee_spec( $ARGS{'emp_spec'} );
+    $status = _process_employee_spec( $ARGS{'emp_spec'} );
     return $status unless $status->ok;
     my $eid = $status->payload->{'eid'};
     die "AAHQ! Could not extract EID from payload: " . Dumper( $status ) unless $eid;
@@ -251,9 +339,9 @@ sub _dump_history {
     if ( $status->ok ) {
         my $pl;
         if ( $type eq 'priv' ) {
-            $pl .= print_priv_history( $status->payload );
+            $pl .= _print_priv_history( $status->payload );
         } elsif ( $type eq 'schedule' ) {
-            $pl .= print_schedule_history( $status->payload );
+            $pl .= _print_schedule_history( $status->payload );
         } else {
             die "AH! bad type " . $type || "undefined";
         }
@@ -264,13 +352,13 @@ sub _dump_history {
 }
 
 
-=head2 print_priv_history
+=head3 _print_priv_history
 
 Take a privhistory and print it
 
 =cut
 
-sub print_priv_history {
+sub _print_priv_history {
     my $props = shift;
     my $eid = $props->{'eid'};
     my $nick = $props->{'nick'};
@@ -295,13 +383,13 @@ sub print_priv_history {
 }
 
 
-=head2 print_schedule_history
+=head3 _print_schedule_history
 
 Take a schedhistory and print it
 
 =cut
 
-sub print_schedule_history {
+sub _print_schedule_history {
     my $props = shift;
     my $eid = $props->{'eid'};
     my $nick = $props->{'nick'};
@@ -334,7 +422,7 @@ sub print_schedule_history {
 }
 
 
-=head2 process_employee_spec
+=head3 _process_employee_spec
 
 Given EMPLOYEE_SPEC, return a status object that can either be OK with
 employee object in payload or NOT_OK with mrest_declare_status already
@@ -342,7 +430,7 @@ called.
 
 =cut
 
-sub process_employee_spec {
+sub _process_employee_spec {
     my $emp_spec = shift;
 
     my ( $eid, $status );
@@ -359,36 +447,4 @@ sub process_employee_spec {
 }
     
 
-=head2 set_history_remark
-
-=cut
-
-sub set_history_remark {
-    print "Entering " . __PACKAGE__ . "::dump_schedule_history\n" if $debug_mode;
-    my ( $ts, $th ) = @_;
-
-    my ( $type, $id_spec, $id );
-    if ( $th->{'PHISTORY_SPEC'} ) {
-        $type = 'priv';
-        $id_spec = 'phid';
-        $id = $th->{'PHISTORY_SPEC'};
-    } elsif ( $th->{'SHISTORY_SPEC'} ) {
-        $type = 'schedule';
-        $id_spec = 'shid';
-        $id = $th->{'SHISTORY_SPEC'};
-    } else {
-        die "Agh~! neither PHISTORY_SPEC nor SHISTORY_SPEC given";
-    }
-
-    my $remark = $th->{'_REST'} || '';
-    $remark =~ s/[\"\']//g;
-
-    my $status = send_req( 'POST', "$type/history/$id_spec/$id", <<"EOS" );
-{ "remark" : "$remark" }
-EOS
-
-    return $status;
-}
-
-
 1;
