@@ -36,7 +36,7 @@ use strict;
 use warnings;
 
 use App::CELL qw( $CELL );
-use App::Dochazka::CLI::HTTP;
+use Web::MREST::CLI::UserAgent;
 
 
 
@@ -50,11 +50,11 @@ App::Dochazka::CLI::Commands::Top - Dispatch targets for top-level commands
 
 =head1 VERSION
 
-Version 0.154
+Version 0.184
 
 =cut
 
-our $VERSION = '0.154';
+our $VERSION = '0.184';
 
 
 
@@ -83,7 +83,7 @@ sub _method_cookiejar {
     my ( $ts, $th ) = @_;
     my $method = $ts->[0];
     
-    my $cookie_jar = App::Dochazka::CLI::HTTP::cookie_jar() || "No cookies in jar";
+    my $cookie_jar = Web::MREST::CLI::UserAgent::cookie_jar() || "No cookies in jar";
 
     return $CELL->status_ok( 'COOKIE_JAR', payload => $cookie_jar );
 }
