@@ -40,8 +40,8 @@ use warnings;
 use App::CELL qw( $CELL );
 use App::Dochazka::CLI qw( $current_emp $debug_mode );
 use App::Dochazka::CLI::Shared qw( print_schedule_object show_current );
-use App::Dochazka::CLI::Util qw( lookup_employee rest_error );
-use App::Dochazka::REST::Model::Schedule;
+use App::Dochazka::CLI::Util qw( lookup_employee parse_test rest_error );
+use App::Dochazka::Model::Schedule;
 use Data::Dumper;
 use Exporter 'import';
 use JSON;
@@ -59,11 +59,11 @@ App::Dochazka::CLI::Commands::Schedule - Schedule commands
 
 =head1 VERSION
 
-Version 0.154
+Version 0.184
 
 =cut
 
-our $VERSION = '0.154';
+our $VERSION = '0.184';
 
 
 
@@ -122,53 +122,104 @@ The functions in this module are called from the parser when it recognizes a com
 =cut
 
 
-=head2 schedule_new
+=head2 Command handlers
 
-Submits the "memsched" (i.e. working schedule stored in memory) to the REST
-server via 'POST submit/new'.
+The routines in this section are called as command handlers.
+
+
+=head3 show_current_schedule 
+
+    SCHEDULE
+    EMPLOYEE_SPEC SCHEDULE
 
 =cut
 
-sub schedule_new {
-    print "Entering " . __PACKAGE__ . "::schedule_new\n" if $debug_mode;
-    my @pl;
+sub show_current_schedule {
+    print "Entering " . __PACKAGE__ . "::show_current_schedule\n" if $debug_mode;
+    my ( $ts, $th ) = @_;
 
-    # make sure there are some memsched entries to begin with
-    if ( ! $memsched ) {
-        return $CELL->status_err( 'DOCHAZKA_CLI_NO_SCHEDULE_ENTRIES_IN_MEMORY' );
-    }
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
 
-    # sort entries by beginning and ending timestamp
-    foreach my $start_str ( sort keys %$memsched ) {
-        foreach my $end_str ( sort @{ $memsched->{$start_str} } ) {
-            push @pl, "[ $start_str, $end_str )";
-        }
-    }
-
-    my $sched = { "schedule" => \@pl };
-    $sched->{'scode'} = $memsched_scode if $memsched_scode;
-    my $json = encode_json $sched;
-
-    my $status = send_req( 'POST', 'schedule/new', $json );
-    if ( $status->ok ) {
-        my $sch_obj = App::Dochazka::REST::Model::Schedule->spawn( %{ $status->payload } );
-        my $pl = '';
-        if ( my $http_status = $status->{'http_status'} ) {
-            $pl .= "HTTP status: $http_status\n";
-        }
-        $pl .= print_schedule_object( $sch_obj );
-        clear_memsched_entries();
-        return $CELL->status_ok( 
-            'DOCHAZKA_CLI_NORMAL_COMPLETION', 
-            http_status => '200 OK',
-            payload => $pl 
-        );
-    }
-    return $status;
+    return show_current( 'schedule', $th );
 }
 
 
-=head2 fetch_all_schedules
+=head3 add_memsched_entry
+
+    SCHEDULE _DOW _TIME _DOW1 _TIME1
+    SCHEDULE _DOW _TIME _HYPHEN _DOW1 _TIME1
+    SCHEDULE _DOW _TIMERANGE
+
+=cut
+
+sub add_memsched_entry {
+    print "Entering " . __PACKAGE__ . "::add_memsched_entry\n" if $debug_mode;
+    my ( $ts, $th ) = @_;
+
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
+
+    my ( $dow_begin, $dow_end, $time_begin, $time_end ) = _canonicalize_th( $th );
+    my ( $date_begin, $date_end ) = ( $dow_map{$dow_begin}, $dow_map{$dow_end} );
+
+    if ( exists( $memsched->{"$date_begin $time_begin"} ) ) {
+        push @{ $memsched->{"$date_begin $time_begin"} }, "$date_end $time_end"
+            unless grep { $_ eq "$date_end $time_end" } @{ $memsched->{"$date_begin $time_begin"} };
+    } else {
+        $memsched->{"$date_begin $time_begin"} = [ "$date_end $time_end" ];
+    }
+
+    print Dumper( $memsched ) if $debug_mode;
+
+    return _dump_memsched_entries();
+}
+
+
+=head3 replicate_memsched_entry
+
+    SCHEDULE ALL _TIMERANGE
+
+Apply timerange to all five days MON-FRI
+
+=cut
+
+sub replicate_memsched_entry {
+    print "Entering " . __PACKAGE__ . "::add_memsched_entry\n" if $debug_mode;
+    my ( $ts, $th ) = @_;
+
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
+
+    foreach my $dow ( qw( MON TUE WED THU FRI ) ) {
+        $th->{_DOW} = $dow;
+        add_memsched_entry( $ts, $th );
+    }
+
+    return _dump_memsched_entries();
+}
+
+
+=head3 clear_memsched_entries
+
+    SCHEDULE CLEAR
+
+=cut
+
+sub clear_memsched_entries {
+    print "Entering " . __PACKAGE__ . "::clear_memsched_entries\n" if $debug_mode;
+    my ( $ts, $th ) = @_;
+
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
+
+    $memsched = {};
+    $memsched_scode = '';
+    return $CELL->status_ok( 'DOCHAZKA_CLI_MEMSCHED_EMPTY' );
+}
+
+
+=head3 fetch_all_schedules
 
     SCHEDULES FETCH ALL 
     SCHEDULES FETCH ALL DISABLED
@@ -181,13 +232,16 @@ sub fetch_all_schedules {
     print "Entering " . __PACKAGE__ . "::fetch_all_schedules\n" if $debug_mode;
     my ( $ts, $th ) = @_;
 
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
+
     my $status = ( $th->{DISABLED} )
         ? send_req( 'GET', 'schedule/all/disabled' )
         : send_req( 'GET', 'schedule/all' );
     if ( $status->ok ) {
         my $pl = '';
         foreach my $sch_hash ( @{ $status->payload } ) {
-            my $sch_obj = App::Dochazka::REST::Model::Schedule->spawn( %$sch_hash );
+            my $sch_obj = App::Dochazka::Model::Schedule->spawn( %$sch_hash );
             $pl .= print_schedule_object( $sch_obj );
             $pl .= "\n";
         }
@@ -197,18 +251,7 @@ sub fetch_all_schedules {
 }
 
 
-=head2 clear_memsched_entries
-
-=cut
-
-sub clear_memsched_entries {
-    $memsched = {};
-    $memsched_scode = '';
-    return $CELL->status_ok( 'DOCHAZKA_CLI_MEMSCHED_EMPTY' );
-}
-
-
-=head2 dump_memsched_entries
+=head3 dump_memsched_entries
 
 Dumps "memsched" (i.e. working schedule stored in memory) to the screen.
 
@@ -217,10 +260,17 @@ illegal and nonsensical schedule in memory, and submit it to the REST
 server. Data integrity controls for new schedule records are performed
 on server-side.
 
+    SCHEDULE DUMP
+    SCHEDULE MEMORY
+
 =cut
 
 sub dump_memsched_entries {
     print "Entering " . __PACKAGE__ . "::dump_memsched_entries\n" if $debug_mode;
+    my ( $ts, $th ) = @_;
+
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
     
     my $pl = '';
 
@@ -244,34 +294,60 @@ sub dump_memsched_entries {
 }
 
 
-=head2 add_memsched_entry
+=head3 schedule_new
 
-    SCHEDULE _DOW _TIME _DOW1 _TIME1
-    SCHEDULE _DOW _TIME _HYPHEN _DOW1 _TIME1
-    SCHEDULE _DOW _TIMERANGE
+Submits the "memsched" (i.e. working schedule stored in memory) to the REST
+server via 'POST submit/new'.
+
+    SCHEDULE NEW
 
 =cut
 
-sub add_memsched_entry {
-    print "Entering " . __PACKAGE__ . "::add_memsched_entry\n" if $debug_mode;
+sub schedule_new {
+    print "Entering " . __PACKAGE__ . "::schedule_new\n" if $debug_mode;
     my ( $ts, $th ) = @_;
-    my ( $dow_begin, $dow_end, $time_begin, $time_end ) = _canonicalize_th( $th );
-    my ( $date_begin, $date_end ) = ( $dow_map{$dow_begin}, $dow_map{$dow_end} );
 
-    if ( exists( $memsched->{"$date_begin $time_begin"} ) ) {
-        push @{ $memsched->{"$date_begin $time_begin"} }, "$date_end $time_end"
-            unless grep { $_ eq "$date_end $time_end" } @{ $memsched->{"$date_begin $time_begin"} };
-    } else {
-        $memsched->{"$date_begin $time_begin"} = [ "$date_end $time_end" ];
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
+
+    my @pl;
+
+    # make sure there are some memsched entries to begin with
+    if ( ! $memsched ) {
+        return $CELL->status_err( 'DOCHAZKA_CLI_NO_SCHEDULE_ENTRIES_IN_MEMORY' );
     }
 
-    print Dumper( $memsched ) if $debug_mode;
+    # sort entries by beginning and ending timestamp
+    foreach my $start_str ( sort keys %$memsched ) {
+        foreach my $end_str ( sort @{ $memsched->{$start_str} } ) {
+            push @pl, "[ $start_str, $end_str )";
+        }
+    }
 
-    return dump_memsched_entries();
+    my $sched = { "schedule" => \@pl };
+    $sched->{'scode'} = $memsched_scode if $memsched_scode;
+    my $json = encode_json $sched;
+
+    my $status = send_req( 'POST', 'schedule/new', $json );
+    if ( $status->ok ) {
+        my $sch_obj = App::Dochazka::Model::Schedule->spawn( %{ $status->payload } );
+        my $pl = '';
+        if ( my $http_status = $status->{'http_status'} ) {
+            $pl .= "HTTP status: $http_status\n";
+        }
+        $pl .= print_schedule_object( $sch_obj );
+        _clear_memsched_entries();
+        return $CELL->status_ok( 
+            'DOCHAZKA_CLI_NORMAL_COMPLETION', 
+            http_status => '200 OK',
+            payload => $pl 
+        );
+    }
+    return $status;
 }
 
 
-=head2 assign_memsched_scode
+=head3 assign_memsched_scode
 
     SCHEDULE SCODE _TERM
 
@@ -283,35 +359,141 @@ sub assign_memsched_scode {
     print "Entering " . __PACKAGE__ . "::assign_memsched_scode\n" if $debug_mode;
     my ( $ts, $th ) = @_;
 
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
+
     $memsched_scode = $th->{_TERM};
 
-    return dump_memsched_entries();
+    return _dump_memsched_entries();
 }
 
 
-=head2 replicate_memsched_entry
+=head3 schedulespec
 
-    SCHEDULE ALL _TIMERANGE
-
-Apply timerange to all five days MON-FRI
+    SCHEDULE_SPEC
+    SCHEDULE_SPEC SHOW
 
 =cut
 
-sub replicate_memsched_entry {
-    print "Entering " . __PACKAGE__ . "::add_memsched_entry\n" if $debug_mode;
+sub schedulespec {
+    print "Entering " . __PACKAGE__ . "::schedulespec\n" if $debug_mode;
     my ( $ts, $th ) = @_;
 
-    foreach my $dow ( qw( MON TUE WED THU FRI ) ) {
-        $th->{_DOW} = $dow;
-        add_memsched_entry( $ts, $th );
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
+
+    my ( $key_spec, $key ) = $th->{'SCHEDULE_SPEC'} =~ m/^(.*)\=(.*)$/;
+
+    my ( $status, $pl );
+    if ( $key_spec =~ m/^sco/i ) {
+        $status = send_req( 'GET', "schedule/scode/$key" );
+    } elsif ( $key_spec =~ m/^sid/ ) {
+        $status = send_req( 'GET', "schedule/sid/$key" );
+    } else {
+        die "AAAHAAAHHH!!! Invalid schedule lookup key " . ( defined( $key_spec ) ? $key_spec : "undefined" )
     }
 
-    return dump_memsched_entries();
+    if ( $status->ok ) {
+        my $sch_obj = App::Dochazka::Model::Schedule->spawn( %{ $status->payload } );
+        $pl = print_schedule_object( $sch_obj );
+        return $CELL->status_ok( "DOCHAZKA_CLI_NORMAL_COMPLETION", payload => $pl );
+    }
+
+    return $status;
 }
 
 
-# the canonical form is "SCHEDULE _DOW _TIME _DOW1 _TIME1"
-# so if we get one of the other forms, we "canonicalize th"
+=head3 schedulespec_remark
+
+    SCHEDULE_SPEC REMARK _TERM
+
+=cut
+
+sub schedulespec_remark {
+    print "Entering " . __PACKAGE__ . "::schedulespec_remark\n" if $debug_mode;
+    my ( $ts, $th ) = @_;
+    
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
+
+    my ( $key_spec, $key ) = $th->{'SCHEDULE_SPEC'} =~ m/^(.*)\=(.*)$/;
+    my $remark = $th->{'_REST'};
+    $remark =~ s/\"/\'/g;
+
+    my $status;
+    if ( $key_spec =~ m/^sco/i ) {
+        $status = send_req( 'PUT', "schedule/scode/$key", <<"EOS" );
+{ "remark" : "$remark" }
+EOS
+    } elsif ( $key_spec =~ m/^sid/ ) {
+        $status = send_req( 'PUT', "schedule/sid/$key", <<"EOS" );
+{ "remark" : "$remark" }
+EOS
+    } else {
+        die "AAAHAAAHHH!!! Invalid schedule lookup key " . ( defined( $key_spec ) ? $key_spec : "undefined" )
+    }
+
+    if ( $status->level eq 'OK' and $status->code eq 'DOCHAZKA_CUD_OK' ) {
+        my $sch_obj = App::Dochazka::Model::Schedule->spawn( %{ $status->payload } );
+        my $pl = print_schedule_object( $sch_obj );
+        return $CELL->status_ok( 'DOCHAZKA_CLI_NORMAL_COMPLETION', payload => $pl );
+    }
+
+    return $status;
+}
+
+=head3 schedulespec_scode
+
+    SCHEDULE_SPEC SCODE _TERM
+
+=cut
+
+sub schedulespec_scode {
+    print "Entering " . __PACKAGE__ . "::schedulespec_scode\n" if $debug_mode;
+    my ( $ts, $th ) = @_;
+    
+    # parse test
+    return parse_test( $ts, $th ) if $ts eq 'PARSE_TEST';
+
+    my ( $key_spec, $key ) = $th->{'SCHEDULE_SPEC'} =~ m/^(.*)\=(.*)$/;
+    my $scode = $th->{'_TERM'};
+
+    my $status;
+    if ( $key_spec =~ m/^sco/i ) {
+        $status = send_req( 'PUT', "schedule/scode/$key", <<"EOS" );
+{ "scode" : "$scode" }
+EOS
+    } elsif ( $key_spec =~ m/^sid/ ) {
+        $status = send_req( 'PUT', "schedule/sid/$key", <<"EOS" );
+{ "scode" : "$scode" }
+EOS
+    } else {
+        die "AAAHAAAHHH!!! Invalid schedule lookup key " . ( defined( $key_spec ) ? $key_spec : "undefined" )
+    }
+
+    if ( $status->level eq 'OK' and $status->code eq 'DOCHAZKA_CUD_OK' ) {
+        my $sch_obj = App::Dochazka::Model::Schedule->spawn( %{ $status->payload } );
+        my $pl = print_schedule_object( $sch_obj );
+        return $CELL->status_ok( 'DOCHAZKA_CLI_NORMAL_COMPLETION', payload => $pl );
+    }
+
+    return $status;
+}
+
+
+
+=head2 Helper functions
+
+Functions called by multiple command handlers
+
+
+=head3 _canonicalize_th
+
+The canonical form is "SCHEDULE _DOW _TIME _DOW1 _TIME1"
+so if we get one of the other forms, we "canonicalize th"
+
+=cut
+
 sub _canonicalize_th {
     my $th = shift;
     print "Entering " . __PACKAGE__ . "::_canonicalize_th with th: " . Dumper( $th ) . "\n" if $debug_mode;
@@ -336,119 +518,29 @@ sub _canonicalize_th {
 }
 
 
-=head2 schedulespec
+=head3 _clear_memsched_entries
 
-    SCHEDULE_SPEC
-    SCHEDULE_SPEC SHOW
+Since clear_memsched_entries is a command handler, if we want to call it from
+within this module we have to use a special argument. Thus we can have our cake
+and eat it, too.
 
 =cut
 
-sub schedulespec {
-    print "Entering " . __PACKAGE__ . "::schedulespec\n" if $debug_mode;
-    my ( $ts, $th ) = @_;
-
-    my ( $key_spec, $key ) = $th->{'SCHEDULE_SPEC'} =~ m/^(.*)\=(.*)$/;
-
-    my ( $status, $pl );
-    if ( $key_spec =~ m/^sco/i ) {
-        $status = send_req( 'GET', "schedule/scode/$key" );
-    } elsif ( $key_spec =~ m/^sid/ ) {
-        $status = send_req( 'GET', "schedule/sid/$key" );
-    } else {
-        die "AAAHAAAHHH!!! Invalid schedule lookup key " . ( defined( $key_spec ) ? $key_spec : "undefined" )
-    }
-
-    if ( $status->ok ) {
-        my $sch_obj = App::Dochazka::REST::Model::Schedule->spawn( %{ $status->payload } );
-        $pl = print_schedule_object( $sch_obj );
-        return $CELL->status_ok( "DOCHAZKA_CLI_NORMAL_COMPLETION", payload => $pl );
-    }
-
-    return $status;
+sub _clear_memsched_entries {
+    return clear_memsched_entries( 'DUMMY_ARG' => 0 );
 }
 
 
-=head2 schedulespec_remark
+=head3 _dump_memsched_entries
 
-    SCHEDULE_SPEC REMARK _TERM
+Since dump_memsched_entries is a command handler, if we want to call it from
+within this module we have to use a special argument. Thus we can have our cake
+and eat it, too.
 
 =cut
 
-sub schedulespec_remark {
-    print "Entering " . __PACKAGE__ . "::schedulespec_remark\n" if $debug_mode;
-    my ( $ts, $th ) = @_;
-    
-    my ( $key_spec, $key ) = $th->{'SCHEDULE_SPEC'} =~ m/^(.*)\=(.*)$/;
-    my $remark = $th->{'_REST'};
-    $remark =~ s/\"/\'/g;
-
-    my $status;
-    if ( $key_spec =~ m/^sco/i ) {
-        $status = send_req( 'PUT', "schedule/scode/$key", <<"EOS" );
-{ "remark" : "$remark" }
-EOS
-    } elsif ( $key_spec =~ m/^sid/ ) {
-        $status = send_req( 'PUT', "schedule/sid/$key", <<"EOS" );
-{ "remark" : "$remark" }
-EOS
-    } else {
-        die "AAAHAAAHHH!!! Invalid schedule lookup key " . ( defined( $key_spec ) ? $key_spec : "undefined" )
-    }
-
-    if ( $status->level eq 'OK' and $status->code eq 'DOCHAZKA_CUD_OK' ) {
-        my $sch_obj = App::Dochazka::REST::Model::Schedule->spawn( %{ $status->payload } );
-        my $pl = print_schedule_object( $sch_obj );
-        return $CELL->status_ok( 'DOCHAZKA_CLI_NORMAL_COMPLETION', payload => $pl );
-    }
-
-    return $status;
-}
-
-=head2 schedulespec_scode
-
-    SCHEDULE_SPEC SCODE _TERM
-
-=cut
-
-sub schedulespec_scode {
-    print "Entering " . __PACKAGE__ . "::schedulespec_scode\n" if $debug_mode;
-    my ( $ts, $th ) = @_;
-    
-    my ( $key_spec, $key ) = $th->{'SCHEDULE_SPEC'} =~ m/^(.*)\=(.*)$/;
-    my $scode = $th->{'_TERM'};
-
-    my $status;
-    if ( $key_spec =~ m/^sco/i ) {
-        $status = send_req( 'PUT', "schedule/scode/$key", <<"EOS" );
-{ "scode" : "$scode" }
-EOS
-    } elsif ( $key_spec =~ m/^sid/ ) {
-        $status = send_req( 'PUT', "schedule/sid/$key", <<"EOS" );
-{ "scode" : "$scode" }
-EOS
-    } else {
-        die "AAAHAAAHHH!!! Invalid schedule lookup key " . ( defined( $key_spec ) ? $key_spec : "undefined" )
-    }
-
-    if ( $status->level eq 'OK' and $status->code eq 'DOCHAZKA_CUD_OK' ) {
-        my $sch_obj = App::Dochazka::REST::Model::Schedule->spawn( %{ $status->payload } );
-        my $pl = print_schedule_object( $sch_obj );
-        return $CELL->status_ok( 'DOCHAZKA_CLI_NORMAL_COMPLETION', payload => $pl );
-    }
-
-    return $status;
-}
-
-
-=head2 show_current_schedule 
-
-=cut
-
-sub show_current_schedule {
-    print "Entering " . __PACKAGE__ . "::show_current_schedule\n" if $debug_mode;
-    my ( $ts, $th ) = @_;
-
-    return show_current( 'schedule', $th );
+sub _dump_memsched_entries {
+    return dump_memsched_entries( 'DUMMY_ARG' => 0 );
 }
 
 
