@@ -42,11 +42,11 @@ App::Dochazka::CLI::Guide - Dochazka CLI Guide
 
 =head1 VERSION
 
-Version 0.154
+Version 0.184
 
 =cut
 
-our $VERSION = '0.154';
+our $VERSION = '0.184';
 
 
 
@@ -190,7 +190,7 @@ Assuming nothing points to it, an activity can be deleted:
 
 =head2 Employees
 
-=head3 Specifying an employee
+=head3 Specify an employee
 
 Wherever it is necessary to specify an employee, the command is shown as 
 
@@ -216,6 +216,32 @@ force a lookup on a particular property:
 Note that there must not be any whitespace before or after the equal sign.
 
 
+=head3 Create a new employee 
+
+If you are using LDAP authentication, your employees are in the LDAP database
+and you can turn on DOCHAZKA_LDAP_AUTOCREATE. Then employees will be created
+automatically the first time they log in. Alternatively, employees can be
+created manually:
+
+    PUT employee nick $NICK { "fullname" : "King George III" }
+
+In addition to "fullname", the following properties can be specified inside
+the curly brackets:
+
+    sec_id
+    email
+    password
+    remark
+
+For example, to add an employee with nick 'nancy' the command might look like
+this:
+
+    PUT employee nick nancy { "fullname" : "Nancy Bright Red", "email" : "nancy@identity.org" }
+   
+The new employee will have passerby privileges only until a privhistory record
+is created (see L<"Add privilege history record">, below).
+
+
 =head3 Set employee secondary ID
 
 To manually change an employee's secondary ID, do one of the following:
@@ -624,6 +650,15 @@ Provided the employee has a schedule defined, it can be displayed by typing:
 
     SCHEDULE HISTORY
 
+
+
+=head2 Intervals
+
+=head3 Enter an interval
+
+    INTERVAL $ACTIVITY_CODE $TIMESTAMP_BEGIN $TIMESTAMP_END
+    INTERVAL $ACTIVITY_CODE $TIMESTAMP_BEGIN - $TIMESTAMP_END
+
 =cut
 
 
